@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,11 +16,11 @@
  */
 
 /*
- * $Id: XMLScanner.hpp 568078 2007-08-21 11:43:25Z amassari $
+ * $Id: XMLScanner.hpp 1800006 2017-06-27 02:40:16Z scantor $
  */
 
-#if !defined(XMLSCANNER_HPP)
-#define XMLSCANNER_HPP
+#if !defined(XERCESC_INCLUDE_GUARD_XMLSCANNER_HPP)
+#define XERCESC_INCLUDE_GUARD_XMLSCANNER_HPP
 
 #include <xercesc/framework/XMLBufferMgr.hpp>
 #include <xercesc/framework/XMLErrorCodes.hpp>
@@ -176,6 +176,7 @@ public :
         , const XMLCh* const        text2 = 0
         , const XMLCh* const        text3 = 0
         , const XMLCh* const        text4 = 0
+
     );
 
     // -----------------------------------------------------------------------
@@ -196,13 +197,6 @@ public :
     virtual const XMLCh* getName() const = 0;
     virtual NameIdPool<DTDEntityDecl>* getEntityDeclPool() = 0;
     virtual const NameIdPool<DTDEntityDecl>* getEntityDeclPool() const = 0;
-    virtual unsigned int resolveQName
-    (
-        const   XMLCh* const        qName
-        ,       XMLBuffer&          prefixBufToFill
-        , const short               mode
-        ,       int&                prefixColonPos
-    ) = 0;
     virtual void scanDocument
     (
         const   InputSource&    src
@@ -215,6 +209,8 @@ public :
         , const bool            toCache = false
     ) = 0;
 
+    virtual void resetCachedGrammar ();
+
     // -----------------------------------------------------------------------
     //  Getter methods
     // -----------------------------------------------------------------------
@@ -254,7 +250,7 @@ public :
     ) const;*/
     const Locator* getLocator() const;
     const ReaderMgr* getReaderMgr() const;
-    unsigned int getSrcOffset() const;
+    XMLFilePos getSrcOffset() const;
     bool getStandalone() const;
     const XMLValidator* getValidator() const;
     XMLValidator* getValidator();
@@ -265,7 +261,9 @@ public :
     XMLCh* getExternalSchemaLocation() const;
     XMLCh* getExternalNoNamespaceSchemaLocation() const;
     SecurityManager* getSecurityManager() const;
+    bool getDisallowDTD() const;
     bool getLoadExternalDTD() const;
+    bool getLoadSchema() const;
     bool getNormalizeData() const;
     bool isCachingGrammarFromParse() const;
     bool isUsingCachedGrammarInParse() const;
@@ -277,12 +275,18 @@ public :
     unsigned int getPrefixId(const XMLCh* const prefix) const;
     const XMLCh* getPrefixForId(unsigned int prefId) const;
 
+    // Return is a reference so that we can return it as void* from
+    // getProperty.
+    //
+    const XMLSize_t& getLowWaterMark() const;
+
     bool getGenerateSyntheticAnnotations() const;
     bool getValidateAnnotations() const;
     bool getIgnoreCachedDTD() const;
     bool getIgnoreAnnotations() const;
     bool getDisableDefaultEntityResolution() const;
     bool getSkipDTDValidation() const;
+    bool getHandleMultipleImports() const;
 
     // -----------------------------------------------------------------------
     //  Getter methods
@@ -347,6 +351,7 @@ public :
     // -----------------------------------------------------------------------
     //  Setter methods
     // -----------------------------------------------------------------------
+    void addGlobalPrefix(const XMLCh* const prefix, const unsigned int uriId);
     void setDocHandler(XMLDocumentHandler* const docHandler);
     void setDocTypeHandler(DocTypeHandler* const docTypeHandler);
     void setDoNamespaces(const bool doNamespaces);
@@ -371,12 +376,15 @@ public :
     void setExternalSchemaLocation(const char* const schemaLocation);
     void setExternalNoNamespaceSchemaLocation(const char* const noNamespaceSchemaLocation);
     void setSecurityManager(SecurityManager* const securityManager);
+    void setDisallowDTD(const bool disallowDTD);
     void setLoadExternalDTD(const bool loadDTD);
+    void setLoadSchema(const bool loadSchema);
     void setNormalizeData(const bool normalizeData);
     void setCalculateSrcOfs(const bool newValue);
     void setParseSettings(XMLScanner* const refScanner);
     void setStandardUriConformant(const bool newValue);
-    void setInputBufferSize(const size_t bufferSize);
+    void setInputBufferSize(const XMLSize_t bufferSize);
+    void setLowWaterMark(XMLSize_t newValue);
 
     void setGenerateSyntheticAnnotations(const bool newValue);
     void setValidateAnnotations(const bool newValue);
@@ -384,19 +392,13 @@ public :
     void setIgnoreAnnotations(const bool newValue);
     void setDisableDefaultEntityResolution(const bool newValue);
     void setSkipDTDValidation(const bool newValue);
+    void setHandleMultipleImports(const bool newValue);
 
     // -----------------------------------------------------------------------
     //  Mutator methods
     // -----------------------------------------------------------------------
     void incrementErrorCount(void);			// For use by XMLValidator
 
-    // -----------------------------------------------------------------------
-    //  Deprecated methods as of 3.2.0. Use getValidationScheme() and
-    //  setValidationScheme() instead.
-    // -----------------------------------------------------------------------
-    bool getDoValidation() const;
-    void setDoValidation(const bool validate);
-
     // -----------------------------------------------------------------------
     //  Document scanning methods
     //
@@ -449,10 +451,15 @@ public :
     );
 
     // -----------------------------------------------------------------------
-    //  Notification that lazy data has been deleted
+    //  Helper methods
     // -----------------------------------------------------------------------
-	static void reinitScannerMutex();
-	static void reinitMsgLoader();
+    unsigned int resolveQName
+    (
+        const   XMLCh* const        qName
+        ,       XMLBuffer&          prefixBufToFill
+        , const ElemStack::MapModes mode
+        ,       int&                prefixColonPos
+    );
 
 protected:
     // -----------------------------------------------------------------------
@@ -492,17 +499,29 @@ protected:
     void checkInternalDTD(bool hasExtSubset, const XMLCh* const sysId, const XMLCh* const pubId);
     void checkIDRefs();
     bool isLegalToken(const XMLPScanToken& toCheck);
-    XMLTokens senseNextToken(unsigned int& orgReader);
+    XMLTokens senseNextToken(XMLSize_t& orgReader);
     void initValidator(XMLValidator* theValidator);
     inline void resetValidationContext();
     unsigned int *getNewUIntPtr();
     void resetUIntPool();
     void recreateUIntPool();
+    unsigned int resolvePrefix
+    (
+        const   XMLCh* const        prefix
+        , const ElemStack::MapModes mode
+    );
+    unsigned int resolveQNameWithColon
+    (
+        const   XMLCh* const        qName
+        ,       XMLBuffer&          prefixBufToFill
+        , const ElemStack::MapModes mode
+        , const int                 prefixColonPos
+    );
 
     inline
     void setAttrDupChkRegistry
          (
-            const unsigned int &attrNumber
+            const XMLSize_t    &attrNumber
           ,       bool         &toUseHashTable
          );
 
@@ -512,6 +531,9 @@ protected:
     //  fBufferSize
     //      Maximum input buffer size
     //
+    //  fLowWaterMark
+    //      The low water mark for the raw byte buffer.
+    //
     //  fAttrList
     //      Every time we get a new element start tag, we have to pass to
     //      the document handler the attributes found. To make it more
@@ -701,9 +723,16 @@ protected:
     //      The number of general entities expanded so far in this document.
     //      Only meaningful when fSecurityManager != null
     //
+    //  fDisallowDTD
+    //      This flag indicates whether the presence of a DTD should be fatal
+    //
     //  fLoadExternalDTD
     //      This flag indicates whether the external DTD be loaded or not
     //
+    //  fLoadSchema
+    //      This flag indicates whether the parser should attempt to load
+    //      schemas if they cannot be found in the grammar pool.
+    //
     //  fNormalizeData
     //      This flag indicates whether the parser should perform datatype
     //      normalization that is defined in the schema.
@@ -716,14 +745,14 @@ protected:
     //      This flag controls whether we force conformant URI
     //
     //  fXMLVersion
-    //      Enum to indicate if the main doc is XML 1.1 or XML 1.0 conformant    
+    //      Enum to indicate if the main doc is XML 1.1 or XML 1.0 conformant
     //  fUIntPool
     //      pool of unsigned integers to help with duplicate attribute
     //      detection and filling in default/fixed attributes
     //  fUIntPoolRow
     //      current row in fUIntPool
     //  fUIntPoolCol
-    //      current column i row
+    //      current column in row
     //  fUIntPoolRowTotal
     //      total number of rows in table
     //
@@ -731,7 +760,8 @@ protected:
     //      Pluggable memory manager for dynamic allocation/deallocation.
     //
     // -----------------------------------------------------------------------
-    size_t                      fBufferSize;
+    XMLSize_t                   fBufferSize;
+    XMLSize_t                   fLowWaterMark;
     bool                        fStandardUriConformant;
     bool                        fCalculateSrcOfs;
     bool                        fDoNamespaces;
@@ -747,7 +777,9 @@ protected:
     bool                        fIdentityConstraintChecking;
     bool                        fToCacheGrammar;
     bool                        fUseCachedGrammar;
+    bool                        fDisallowDTD;
     bool                        fLoadExternalDTD;
+    bool                        fLoadSchema;
     bool                        fNormalizeData;
     bool                        fGenerateSyntheticAnnotations;
     bool                        fValidateAnnotations;
@@ -755,9 +787,10 @@ protected:
     bool                        fIgnoreAnnotations;
     bool                        fDisableDefaultEntityResolution;
     bool                        fSkipDTDValidation;
+    bool                        fHandleMultipleImports;
     int                         fErrorCount;
-    unsigned int                fEntityExpansionLimit;
-    unsigned int                fEntityExpansionCount;
+    XMLSize_t                   fEntityExpansionLimit;
+    XMLSize_t                   fEntityExpansionCount;
     unsigned int                fEmptyNamespaceId;
     unsigned int                fUnknownNamespaceId;
     unsigned int                fXMLNamespaceId;
@@ -770,7 +803,7 @@ protected:
     XMLUInt32                   fScannerId;
     XMLUInt32                   fSequenceId;
     RefVectorOf<XMLAttr>*       fAttrList;
-    RefHash2KeysTableOf<XMLAttr>*  fAttrDupChkRegistry;    
+    RefHash2KeysTableOf<XMLAttr>*  fAttrDupChkRegistry;
     XMLDocumentHandler*         fDocHandler;
     DocTypeHandler*             fDocTypeHandler;
     XMLEntityHandler*           fEntityHandler;
@@ -822,7 +855,7 @@ private :
     //  Private scanning methods
     // -----------------------------------------------------------------------
     bool getQuotedString(XMLBuffer& toFill);
-    unsigned int scanUpToWSOr
+    XMLSize_t scanUpToWSOr
     (
                 XMLBuffer&  toFill
         , const XMLCh       chEndChar
@@ -943,7 +976,7 @@ inline const ReaderMgr* XMLScanner::getReaderMgr() const
     return &fReaderMgr;
 }
 
-inline unsigned int XMLScanner::getSrcOffset() const
+inline XMLFilePos XMLScanner::getSrcOffset() const
 {
     return fReaderMgr.getSrcOffset();
 }
@@ -1043,11 +1076,21 @@ inline SecurityManager* XMLScanner::getSecurityManager() const
     return fSecurityManager;
 }
 
+inline bool XMLScanner::getDisallowDTD() const
+{
+    return fDisallowDTD;
+}
+
 inline bool XMLScanner::getLoadExternalDTD() const
 {
     return fLoadExternalDTD;
 }
 
+inline bool XMLScanner::getLoadSchema() const
+{
+    return fLoadSchema;
+}
+
 inline bool XMLScanner::getNormalizeData() const
 {
     return fNormalizeData;
@@ -1113,6 +1156,11 @@ inline bool XMLScanner::getValidateAnnotations() const
     return fValidateAnnotations;
 }
 
+inline const XMLSize_t& XMLScanner::getLowWaterMark() const
+{
+    return fLowWaterMark;
+}
+
 inline bool XMLScanner::getIgnoreCachedDTD() const
 {
     return fIgnoreCachedDTD;
@@ -1133,9 +1181,19 @@ inline bool XMLScanner::getSkipDTDValidation() const
     return fSkipDTDValidation;
 }
 
+inline bool XMLScanner::getHandleMultipleImports() const
+{
+    return fHandleMultipleImports;
+}
+
 // ---------------------------------------------------------------------------
 //  XMLScanner: Setter methods
 // ---------------------------------------------------------------------------
+inline void XMLScanner::addGlobalPrefix(const XMLCh* const prefix, const unsigned int uriId)
+{
+    fElemStack.addGlobalPrefix(prefix, uriId);
+}
+
 inline void XMLScanner::setDocHandler(XMLDocumentHandler* const docHandler)
 {
     fDocHandler = docHandler;
@@ -1248,18 +1306,28 @@ inline void XMLScanner::setExternalNoNamespaceSchemaLocation(const char* const n
 inline void XMLScanner::setSecurityManager(SecurityManager* const securityManager)
 {
     fSecurityManager = securityManager;
-    if(securityManager != 0) 
+    if(securityManager != 0)
     {
         fEntityExpansionLimit = securityManager->getEntityExpansionLimit();
         fEntityExpansionCount = 0;
     }
 }
 
+inline void XMLScanner::setDisallowDTD(const bool disallowDTD)
+{
+    fDisallowDTD = disallowDTD;
+}
+
 inline void XMLScanner::setLoadExternalDTD(const bool loadDTD)
 {
     fLoadExternalDTD = loadDTD;
 }
 
+inline void XMLScanner::setLoadSchema(const bool loadSchema)
+{
+    fLoadSchema = loadSchema;
+}
+
 inline void XMLScanner::setNormalizeData(const bool normalizeData)
 {
     fNormalizeData = normalizeData;
@@ -1296,10 +1364,15 @@ inline void XMLScanner::setValidateAnnotations(const bool newValue)
     fValidateAnnotations = newValue;
 }
 
-inline void XMLScanner::setInputBufferSize(const size_t bufferSize)
+inline void XMLScanner::setInputBufferSize(const XMLSize_t bufferSize)
 {
     fBufferSize = bufferSize;
-    fCDataBuf.setFullHandler(this, (unsigned int)fBufferSize);
+    fCDataBuf.setFullHandler(this, fBufferSize);
+}
+
+inline void XMLScanner::setLowWaterMark(XMLSize_t newValue)
+{
+    fLowWaterMark = newValue;
 }
 
 inline void XMLScanner::setIgnoredCachedDTD(const bool newValue)
@@ -1322,6 +1395,11 @@ inline void XMLScanner::setSkipDTDValidation(const bool newValue)
     fSkipDTDValidation = newValue;
 }
 
+inline void XMLScanner::setHandleMultipleImports(const bool newValue)
+{
+    fHandleMultipleImports = newValue;
+}
+
 // ---------------------------------------------------------------------------
 //  XMLScanner: Mutator methods
 // ---------------------------------------------------------------------------
@@ -1330,23 +1408,6 @@ inline void XMLScanner::incrementErrorCount()
     ++fErrorCount;
 }
 
-// ---------------------------------------------------------------------------
-//  XMLScanner: Deprecated methods
-// ---------------------------------------------------------------------------
-inline bool XMLScanner::getDoValidation() const
-{
-    return fValidate;
-}
-
-inline void XMLScanner::setDoValidation(const bool validate)
-{
-    fValidate = validate;
-    if (fValidate)
-        fValScheme = Val_Always;
-    else
-        fValScheme = Val_Never;
-}
-
 inline void XMLScanner::resetValidationContext()
 {
     fValidationContext->clearIdRefList();
@@ -1354,7 +1415,7 @@ inline void XMLScanner::resetValidationContext()
     fEntityDeclPoolRetrieved = false;
 }
 
-inline void XMLScanner::setAttrDupChkRegistry(const unsigned int &attrNumber
+inline void XMLScanner::setAttrDupChkRegistry(const XMLSize_t    &attrNumber
                                             ,       bool         &toUseHashTable)
 {
    // once the attribute exceed 100, we use hash table to check duplication
@@ -1366,7 +1427,7 @@ inline void XMLScanner::setAttrDupChkRegistry(const unsigned int &attrNumber
         {
             fAttrDupChkRegistry = new (fMemoryManager) RefHash2KeysTableOf<XMLAttr>
             (
-              2*attrNumber+1, false, new (fMemoryManager)HashXMLCh(), fMemoryManager
+              2*attrNumber+1, false, fMemoryManager
             );
         }
         else
@@ -1385,4 +1446,3 @@ inline Grammar::GrammarType XMLScanner::getCurrentGrammarType() const
 XERCES_CPP_NAMESPACE_END
 
 #endif
-
