@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,16 +16,17 @@
  */
 
 /*
- * $Id: SAX2XMLReader.hpp 569031 2007-08-23 15:05:28Z amassari $
+ * $Id: SAX2XMLReader.hpp 983783 2010-08-09 19:20:46Z amassari $
  */
 
-#ifndef SAX2XMLReader_HPP
-#define SAX2XMLReader_HPP
+#if !defined(XERCESC_INCLUDE_GUARD_SAX2XMLREADER_HPP)
+#define XERCESC_INCLUDE_GUARD_SAX2XMLREADER_HPP
 
 #include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/framework/XMLPScanToken.hpp>
+#include <xercesc/validators/common/Grammar.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -37,7 +38,6 @@ class InputSource;
 class LexicalHandler;
 class DeclHandler;
 class XMLDocumentHandler;
-class Grammar;
 
 class SAX2_EXPORT SAX2XMLReader
 {
@@ -53,7 +53,19 @@ public:
       *    Val_Always: The parser will always report validation errors.
       *    Val_Auto:   The parser will report validation errors only if a grammar is specified.
       *
-      * @see #setValidationScheme
+      * The schemes map to these feature values:
+      *    Val_Never:
+      *        parser->setFeature(XMLUni::fgSAX2CoreValidation, false);
+      *
+      *    Val_Always:
+      *        parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
+      *        parser->setFeature(XMLUni::fgXercesDynamic, false);
+      *
+      *    Val_Auto:
+      *        parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
+      *        parser->setFeature(XMLUni::fgXercesDynamic, true);
+      *
+      * @see #setFeature
       */
     enum ValSchemes
     {
@@ -113,26 +125,25 @@ public:
       */
     virtual ErrorHandler* getErrorHandler() const = 0 ;
 
-	/**
-     * Query the current state of any feature in a SAX2 XMLReader.
-	  *
-	  * @param name The unique identifier (URI) of the feature being set.
-	  * @return The current state of the feature.
-     * @exception SAXNotRecognizedException If the requested feature is not known.
-	  */
-	virtual bool getFeature(const XMLCh* const name) const = 0;
+    /**
+      * Query the current state of any feature in a SAX2 XMLReader.
+      *
+      * @param name The unique identifier (URI) of the feature being set.
+      * @return The current state of the feature.
+      * @exception SAXNotRecognizedException If the requested feature is not known.
+      */
+    virtual bool getFeature(const XMLCh* const name) const = 0;
 
-	/**
+   /**
      * Query the current value of a property in a SAX2 XMLReader.
      *
      * The parser owns the returned pointer.  The memory allocated for
      * the returned pointer will be destroyed when the parser is deleted.
      *
-     * To ensure assessiblity of the returned information after the parser
+     * To ensure accessibility of the returned information after the parser
      * is deleted, callers need to copy and store the returned information
      * somewhere else; otherwise you may get unexpected result.  Since the returned
-     * pointer is a generic void pointer, see
-     * http://xerces.apache.org/xerces-c/program-sax2.html#SAX2Properties to learn
+     * pointer is a generic void pointer, see the SAX2 Programming Guide to learn
      * exactly what type of property value each property returns for replication.
      *
      * @param name The unique identifier (URI) of the property being set.
@@ -156,7 +167,7 @@ public:
     * handler immediately.
     *
     * @param handler The document handler.
-    * @see DocumentHandler#DocumentHandler
+    * @see ContentHandler#ContentHandler
     * @see HandlerBase#HandlerBase
     */
     virtual void setContentHandler(ContentHandler* const handler) = 0;
@@ -218,7 +229,7 @@ public:
   /**
     * Set the state of any feature in a SAX2 XMLReader.
     * Supported features in SAX2 for xerces-c are:
-    * <br>(See http://xerces.apache.org/xerces-c/program-sax2.html#SAX2Features for detail description).
+    * <br>(See the SAX2 Programming Guide for detail description).
     *
     * <br>http://xml.org/sax/features/validation (default: true)
     * <br>http://xml.org/sax/features/namespaces (default: true)
@@ -227,10 +238,10 @@ public:
     * <br>http://apache.org/xml/features/validation/reuse-grammar (default: false)
     * <br>http://apache.org/xml/features/validation/schema (default: true)
     * <br>http://apache.org/xml/features/validation/schema-full-checking (default: false)
+    * <br>http://apache.org/xml/features/validating/load-schema (default: true)
     * <br>http://apache.org/xml/features/nonvalidating/load-external-dtd (default: true)
     * <br>http://apache.org/xml/features/continue-after-fatal-error (default: false)
     * <br>http://apache.org/xml/features/validation-error-as-fatal (default: false)
-    * <br>http://apache.org/xml/features/validation/reuse-validator (Deprecated) (default: false)
     *
     * @param name The unique identifier (URI) of the feature.
     * @param value The requested state of the feature (true or false).
@@ -243,20 +254,23 @@ public:
   /**
     * Set the value of any property in a SAX2 XMLReader.
     * Supported properties in SAX2 for xerces-c are:
-    * <br>(See http://xerces.apache.org/xerces-c/program-sax2.html#SAX2Properties for detail description).
+    * <br>(See the SAX2 Programming Guide for detail description).
     *
     * <br>http://apache.org/xml/properties/schema/external-schemaLocation
-    * <br>http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation.
+    * <br>http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation
+    * <br>http://apache.org/xml/properties/security-manager
+    * <br>http://apache.org/xml/properties/low-water-mark
+    * <br>http://apache.org/xml/properties/scannerName
     *
     * It takes a void pointer as the property value.  Application is required to initialize this void
-    * pointer to a correct type.  See http://xerces.apache.org/xerces-c/program-sax2.html#SAX2Properties
+    * pointer to a correct type.  See the SAX2 Programming Guide
     * to learn exactly what type of property value each property expects for processing.
     * Passing a void pointer that was initialized with a wrong type will lead to unexpected result.
     * If the same property is set more than once, the last one takes effect.
     *
     * @param name The unique identifier (URI) of the property being set.
     * @param value The requested value for the property.  See
-    *            http://xerces.apache.org/xerces-c/program-sax2.html#SAX2Properties to learn
+    *            the SAX2 Programming Guide to learn
     *            exactly what type of property value each property expects for processing.
     *            Passing a void pointer that was initialized with a wrong type will lead
     *            to unexpected result.
@@ -287,7 +301,7 @@ public:
     * @see InputSource#InputSource
     * @see #setEntityResolver
     * @see #setDTDHandler
-    * @see #setDocumentHandler
+    * @see #setContentHandler
     * @see #setErrorHandler
     */
     virtual void parse
@@ -312,7 +326,7 @@ public:
     *            wrapping another exception.
     * @exception XMLException An exception from the parser or client
     *            handler code.
-    * @see #parse(InputSource)
+    * @see #parse(const InputSource&)
     */
     virtual void parse
     (
@@ -336,13 +350,13 @@ public:
     *            wrapping another exception.
     * @exception XMLException An exception from the parser or client
     *            handler code.
-    * @see #parse(InputSource)
+    * @see #parse(const InputSource&)
     */
     virtual void parse
     (
         const   char* const     systemId
     ) = 0;
-	
+
     //@}
 
     // -----------------------------------------------------------------------
@@ -429,7 +443,7 @@ public:
       * @return number of errors encountered during the latest
       *			parse operation.
       */
-    virtual int getErrorCount() const = 0 ;
+    virtual XMLSize_t getErrorCount() const = 0 ;
 
     /**
       * This method returns the state of the parser's
@@ -457,7 +471,7 @@ public:
       *         set validation constraint errors as fatal, false
       *         otherwise.
       *
-      * @see #setValidationContraintFatal
+      * @see #setValidationConstraintFatal
       * @see #getFeature
       */
     virtual bool getValidationConstraintFatal() const = 0;
@@ -491,7 +505,7 @@ public:
       *
       * @return offset within the input source
       */
-    virtual unsigned int getSrcOffset() const = 0;
+    virtual XMLFilePos getSrcOffset() const = 0;
 
     //@}
 
@@ -501,15 +515,15 @@ public:
     /** @name Setter Methods (Xerces-C specific) */
     //@{
     /**
-	  * This method is used to set a validator.
-	  *
-	  * <b>SAX2XMLReader assumes responsibility for the validator.  It will be
-	  * deleted when the XMLReader is destroyed.</b>
-	  *
-	  * @param valueToAdopt A pointer to the validator that the reader should use.
-	  *
-	  */
-	virtual void setValidator(XMLValidator* valueToAdopt) = 0;
+      * This method is used to set a validator.
+      *
+      * <b>SAX2XMLReader assumes responsibility for the validator.  It will be
+      * deleted when the XMLReader is destroyed.</b>
+      *
+      * @param valueToAdopt A pointer to the validator that the reader should use.
+      *
+      */
+    virtual void setValidator(XMLValidator* valueToAdopt) = 0;
 
     /**
       * This method allows users to set the parser's behaviour when it
@@ -536,7 +550,7 @@ public:
 
     /**
       * This method allows users to set the parser's behaviour when it
-      * encounters a validtion constraint error. If set to true, and the
+      * encounters a validation constraint error. If set to true, and the
       * the parser will treat validation error as fatal and will exit depends on the
       * state of "getExitOnFirstFatalError". If false, then it will
       * report the error and continue processing.
@@ -582,7 +596,7 @@ public:
       * the scan of the prolog failed and the token is not going to work on
       * subsequent scanNext() calls.
       *
-      * @param systemId A pointer to a Unicode string represting the path
+      * @param systemId A pointer to a Unicode string representing the path
       *                 to the XML file to be parsed.
       * @param toFill   A token maintaing state information to maintain
       *                 internal consistency between invocation of 'parseNext'
@@ -615,10 +629,10 @@ public:
       * the scan of the prolog failed and the token is not going to work on
       * subsequent scanNext() calls.
       *
-      * @param systemId A pointer to a regular native string represting
+      * @param systemId A pointer to a regular native string representing
       *                 the path to the XML file to be parsed.
       * @param toFill   A token maintaing state information to maintain
-      *                 internal consIstency between invocation of 'parseNext'
+      *                 internal consistency between invocation of 'parseNext'
       *                 calls.
       *
       * @return 'true', if successful in parsing the prolog. It indicates the
@@ -735,13 +749,12 @@ public:
       * is enabled, the parser will cache the grammars for re-use. If a grammar
       * key is found in the pool, no caching of any grammar will take place.
       *
-      * <p><b>"Experimental - subject to change"</b></p>
       *
       * @param source A const reference to the SAX InputSource object which
       *               points to the schema grammar file to be preparsed.
       * @param grammarType The grammar type (Schema or DTD).
       * @param toCache If <code>true</code>, we cache the preparsed grammar,
-      *                otherwise, no chaching. Default is <code>false</code>.
+      *                otherwise, no caching. Default is <code>false</code>.
       * @return The preparsed schema grammar object (SchemaGrammar or
       *         DTDGrammar). That grammar object is owned by the parser.
       *
@@ -754,7 +767,7 @@ public:
       * @see InputSource#InputSource
       */
     virtual Grammar* loadGrammar(const InputSource& source,
-                                 const short grammarType,
+                                 const Grammar::GrammarType grammarType,
                                  const bool toCache = false) = 0;
 
     /**
@@ -765,14 +778,13 @@ public:
       * is enabled, the parser will cache the grammars for re-use. If a grammar
       * key is found in the pool, no caching of any grammar will take place.
       *
-      * <p><b>"Experimental - subject to change"</b></p>
       *
       * @param systemId A const XMLCh pointer to the Unicode string which
       *                 contains the path to the XML grammar file to be
       *                 preparsed.
       * @param grammarType The grammar type (Schema or DTD).
       * @param toCache If <code>true</code>, we cache the preparsed grammar,
-      *                otherwise, no chaching. Default is <code>false</code>.
+      *                otherwise, no caching. Default is <code>false</code>.
       * @return The preparsed schema grammar object (SchemaGrammar or
       *         DTDGrammar). That grammar object is owned by the parser.
       *
@@ -783,7 +795,7 @@ public:
       * @exception DOMException A DOM exception as per DOM spec.
       */
     virtual Grammar* loadGrammar(const XMLCh* const systemId,
-                                 const short grammarType,
+                                 const Grammar::GrammarType grammarType,
                                  const bool toCache = false) = 0;
 
     /**
@@ -794,13 +806,12 @@ public:
       * is enabled, the parser will cache the grammars for re-use. If a grammar
       * key is found in the pool, no caching of any grammar will take place.
       *
-      * <p><b>"Experimental - subject to change"</b></p>
       *
       * @param systemId A const char pointer to a native string which contains
       *                 the path to the XML grammar file to be preparsed.
       * @param grammarType The grammar type (Schema or DTD).
       * @param toCache If <code>true</code>, we cache the preparsed grammar,
-      *                otherwise, no chaching. Default is <code>false</code>.
+      *                otherwise, no caching. Default is <code>false</code>.
       * @return The preparsed schema grammar object (SchemaGrammar or
       *         DTDGrammar). That grammar object is owned by the parser.
       *
@@ -811,7 +822,7 @@ public:
       * @exception DOMException A DOM exception as per DOM spec.
       */
     virtual Grammar* loadGrammar(const char* const systemId,
-                                 const short grammarType,
+                                 const Grammar::GrammarType grammarType,
                                  const bool toCache = false) = 0;
 
     /**
@@ -829,7 +840,7 @@ public:
       *
       * @param bufferSize The maximum input buffer size
       */
-    virtual void setInputBufferSize(const size_t bufferSize);
+    virtual void setInputBufferSize(const XMLSize_t bufferSize);
 
     //@}
 
@@ -876,7 +887,7 @@ private :
 
 };
 
-inline void SAX2XMLReader::setInputBufferSize(const size_t /*bufferSize*/) 
+inline void SAX2XMLReader::setInputBufferSize(const XMLSize_t /*bufferSize*/)
 {
 }
 
