@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,7 +16,7 @@
  */
 
 /*
- * $Id: RefHashTableOf.c 568078 2007-08-21 11:43:25Z amassari $
+ * $Id: RefHashTableOf.c 678409 2008-07-21 13:08:10Z borisk $
  */
 
 
@@ -28,8 +28,8 @@
 #endif
 
 #include <xercesc/util/Janitor.hpp>
+#include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/NullPointerException.hpp>
-#include <assert.h>
 #include <new>
 
 XERCES_CPP_NAMESPACE_BEGIN
@@ -37,48 +37,10 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Constructors and Destructor
 // ---------------------------------------------------------------------------
-template <class TVal>
-RefHashTableOf<TVal>::RefHashTableOf( const unsigned int modulus
-                                    , const bool adoptElems
-                                    , MemoryManager* const manager)
-
-    : fMemoryManager(manager)
-    , fAdoptedElems(adoptElems)
-    , fBucketList(0)
-    , fHashModulus(modulus)
-    , fInitialModulus(modulus)
-    , fCount(0)
-    , fHash(0)
-
-{
-    initialize(modulus);
-	
-	// create default hasher
-	fHash = new (fMemoryManager) HashXMLCh();
-}
-
-template <class TVal>
-RefHashTableOf<TVal>::RefHashTableOf( const unsigned int modulus
-                                    , const bool adoptElems
-                                    , HashBase* hashBase
-                                    , MemoryManager* const manager)
-
-    : fMemoryManager(manager)
-    , fAdoptedElems(adoptElems)
-    , fBucketList(0)
-    , fHashModulus(modulus)
-    , fInitialModulus(modulus)
-    , fCount(0)
-    , fHash(0)
-{
-    initialize(modulus);
-    // set hasher
-    fHash = hashBase;
-}
-
-template <class TVal>
-RefHashTableOf<TVal>::RefHashTableOf(const unsigned int modulus
-                                     , MemoryManager* const manager)
+template <class TVal, class THasher>
+RefHashTableOf<TVal, THasher>::RefHashTableOf(
+  const XMLSize_t modulus,
+  MemoryManager* const manager)
 
     : fMemoryManager(manager)
     , fAdoptedElems(true)
@@ -86,15 +48,64 @@ RefHashTableOf<TVal>::RefHashTableOf(const unsigned int modulus
     , fHashModulus(modulus)
     , fInitialModulus(modulus)
     , fCount(0)
-    , fHash(0)
 {
     initialize(modulus);
-
-    // create default hasher
-    fHash = new (fMemoryManager) HashXMLCh();
 }
 
-template <class TVal> void RefHashTableOf<TVal>::initialize(const unsigned int modulus)
+template <class TVal, class THasher>
+RefHashTableOf<TVal, THasher>::RefHashTableOf(
+  const XMLSize_t modulus,
+  const THasher& hasher,
+  MemoryManager* const manager)
+
+    : fMemoryManager(manager)
+    , fAdoptedElems(true)
+    , fBucketList(0)
+    , fHashModulus(modulus)
+    , fInitialModulus(modulus)
+    , fCount(0)
+    , fHasher (hasher)
+{
+    initialize(modulus);
+}
+
+template <class TVal, class THasher>
+RefHashTableOf<TVal, THasher>::RefHashTableOf(
+  const XMLSize_t modulus,
+  const bool adoptElems,
+  MemoryManager* const manager)
+
+    : fMemoryManager(manager)
+    , fAdoptedElems(adoptElems)
+    , fBucketList(0)
+    , fHashModulus(modulus)
+    , fInitialModulus(modulus)
+    , fCount(0)
+
+{
+    initialize(modulus);
+}
+
+template <class TVal, class THasher>
+RefHashTableOf<TVal, THasher>::RefHashTableOf(
+  const XMLSize_t modulus,
+  const bool adoptElems,
+  const THasher& hasher,
+  MemoryManager* const manager)
+
+    : fMemoryManager(manager)
+    , fAdoptedElems(adoptElems)
+    , fBucketList(0)
+    , fHashModulus(modulus)
+    , fInitialModulus(modulus)
+    , fCount(0)
+    , fHasher (hasher)
+{
+    initialize(modulus);
+}
+
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::initialize(const XMLSize_t modulus)
 {
     if (modulus == 0)
         ThrowXMLwithMemMgr(IllegalArgumentException, XMLExcepts::HshTbl_ZeroModulus, fMemoryManager);
@@ -103,12 +114,13 @@ template <class TVal> void RefHashTableOf<TVal>::initialize(const unsigned int m
     fBucketList = (RefHashTableBucketElem<TVal>**) fMemoryManager->allocate
     (
         fHashModulus * sizeof(RefHashTableBucketElem<TVal>*)
-    ); //new RefHashTableBucketElem<TVal>*[fHashModulus];
-    for (unsigned int index = 0; index < fHashModulus; index++)
+    );
+    for (XMLSize_t index = 0; index < fHashModulus; index++)
         fBucketList[index] = 0;
 }
 
-template <class TVal> RefHashTableOf<TVal>::~RefHashTableOf()
+template <class TVal, class THasher>
+RefHashTableOf<TVal, THasher>::~RefHashTableOf()
 {
     cleanup();
 }
@@ -117,25 +129,26 @@ template <class TVal> RefHashTableOf<TVal>::~RefHashTableOf()
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Element management
 // ---------------------------------------------------------------------------
-template <class TVal> bool RefHashTableOf<TVal>::isEmpty() const
+template <class TVal, class THasher>
+inline bool RefHashTableOf<TVal, THasher>::isEmpty() const
 {
     return fCount==0;
 }
 
-template <class TVal> bool RefHashTableOf<TVal>::
-containsKey(const void* const key) const
+template <class TVal, class THasher>
+inline bool RefHashTableOf<TVal, THasher>::containsKey(const void* const key) const
 {
-    unsigned int hashVal;
+    XMLSize_t hashVal;
     const RefHashTableBucketElem<TVal>* findIt = findBucketElem(key, hashVal);
     return (findIt != 0);
 }
 
-template <class TVal> void RefHashTableOf<TVal>::
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::
 removeKey(const void* const key)
 {
     // Hash the key
-    unsigned int hashVal = fHash->getHashVal(key, fHashModulus, fMemoryManager);
-    assert(hashVal < fHashModulus);
+    XMLSize_t hashVal = fHasher.getHashVal(key, fHashModulus);
 
     //
     //  Search the given bucket for this key. Keep up with the previous
@@ -146,7 +159,7 @@ removeKey(const void* const key)
 
     while (curElem)
     {
-        if (fHash->equals(key, curElem->fKey))
+        if (fHasher.equals(key, curElem->fKey))
         {
             if (!lastElem)
             {
@@ -167,10 +180,9 @@ removeKey(const void* const key)
                 delete curElem->fData;
 
             // Then delete the current element and move forward
- 	        // delete curElem;
+            // delete curElem;
             // destructor doesn't do anything...
-			// curElem->~RefHashTableBucketElem();
-            fMemoryManager->deallocate(curElem);            
+            fMemoryManager->deallocate(curElem);
 
             fCount--;
 
@@ -186,13 +198,14 @@ removeKey(const void* const key)
     ThrowXMLwithMemMgr(NoSuchElementException, XMLExcepts::HshTbl_NoSuchKeyExists, fMemoryManager);
 }
 
-template <class TVal> void RefHashTableOf<TVal>::removeAll()
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::removeAll()
 {
     if(isEmpty())
         return;
 
     // Clean up the buckets first
-    for (unsigned int buckInd = 0; buckInd < fHashModulus; buckInd++)
+    for (XMLSize_t buckInd = 0; buckInd < fHashModulus; buckInd++)
     {
         // Get the bucket list head for this entry
         RefHashTableBucketElem<TVal>* curElem = fBucketList[buckInd];
@@ -210,10 +223,10 @@ template <class TVal> void RefHashTableOf<TVal>::removeAll()
                 delete curElem->fData;
 
             // Then delete the current element and move forward
- 	        // delete curElem;
+             // delete curElem;
             // destructor doesn't do anything...
-			// curElem->~RefHashTableBucketElem();
-            fMemoryManager->deallocate(curElem);            
+            // curElem->~RefHashTableBucketElem();
+            fMemoryManager->deallocate(curElem);
             curElem = nextElem;
         }
 
@@ -228,13 +241,12 @@ template <class TVal> void RefHashTableOf<TVal>::removeAll()
 // now owns the returned data (case of hashtable adopting the data).
 // This function is called by transferElement so that the undeleted data can be transferred
 // to a new key which will own that data.
-template <class TVal> TVal* RefHashTableOf<TVal>::
+template <class TVal, class THasher> TVal* RefHashTableOf<TVal, THasher>::
 orphanKey(const void* const key)
 {
     // Hash the key
     TVal* retVal = 0;
-    unsigned int hashVal = fHash->getHashVal(key, fHashModulus, fMemoryManager);
-    assert(hashVal < fHashModulus);
+    XMLSize_t hashVal = fHasher.getHashVal(key, fHashModulus);
 
     //
     //  Search the given bucket for this key. Keep up with the previous
@@ -245,7 +257,7 @@ orphanKey(const void* const key)
 
     while (curElem)
     {
-        if (fHash->equals(key, curElem->fKey))
+        if (fHasher.equals(key, curElem->fKey))
         {
             if (!lastElem)
             {
@@ -263,8 +275,8 @@ orphanKey(const void* const key)
             // Delete the current element
             // delete curElem;
             // destructor doesn't do anything...
-			// curElem->~RefHashTableBucketElem();
-            fMemoryManager->deallocate(curElem);            
+            // curElem->~RefHashTableBucketElem();
+            fMemoryManager->deallocate(curElem);
             break;
         }
 
@@ -285,14 +297,14 @@ orphanKey(const void* const key)
 //   similar to destructor
 //   called to cleanup the memory, in case destructor cannot be called
 //
-template <class TVal> void RefHashTableOf<TVal>::cleanup()
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::cleanup()
 {
     removeAll();
 
     // Then delete the bucket list & hasher
-    fMemoryManager->deallocate(fBucketList); //delete [] fBucketList;
+    fMemoryManager->deallocate(fBucketList);
     fBucketList = 0;
-    delete fHash;
 }
 
 //
@@ -300,18 +312,16 @@ template <class TVal> void RefHashTableOf<TVal>::cleanup()
 //   similar to constructor
 //   called to re-construct the fElemList from scratch again
 //
-template <class TVal> void RefHashTableOf<TVal>::reinitialize(HashBase* hashBase)
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::reinitialize(const THasher& hasher)
 {
-    if (fBucketList || fHash)
+    if (fBucketList)
         cleanup();
 
+    fHasher = hasher;
+
     fHashModulus = fInitialModulus;
     initialize(fHashModulus);
-
-    if (hashBase)
-        fHash = hashBase;
-    else
-        fHash = new (fMemoryManager) HashXMLCh();   // create default hasher
 }
 
 
@@ -324,7 +334,8 @@ template <class TVal> void RefHashTableOf<TVal>::reinitialize(HashBase* hashBase
 // except that the data is not deleted in "removeKey" even it is adopted so that it
 // can be transferred to key2.
 // whatever key2 has originally will be purged (if adopted)
-template <class TVal> void RefHashTableOf<TVal>::transferElement(const void* const key1, void* key2)
+template <class TVal, class THasher>
+inline void RefHashTableOf<TVal, THasher>::transferElement(const void* const key1, void* key2)
 {
     put(key2, orphanKey(key1));
 }
@@ -333,39 +344,37 @@ template <class TVal> void RefHashTableOf<TVal>::transferElement(const void* con
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Getters
 // ---------------------------------------------------------------------------
-template <class TVal> TVal* RefHashTableOf<TVal>::get(const void* const key)
+template <class TVal, class THasher>
+inline TVal* RefHashTableOf<TVal, THasher>::get(const void* const key)
 {
-    unsigned int hashVal;
+    XMLSize_t hashVal;
     RefHashTableBucketElem<TVal>* findIt = findBucketElem(key, hashVal);
-    if (!findIt)
-        return 0;
-    return findIt->fData;
+    return findIt ? findIt->fData : 0;
 }
 
-template <class TVal> const TVal* RefHashTableOf<TVal>::
+template <class TVal, class THasher>
+inline const TVal* RefHashTableOf<TVal, THasher>::
 get(const void* const key) const
 {
-    unsigned int hashVal;
+    XMLSize_t hashVal;
     const RefHashTableBucketElem<TVal>* findIt = findBucketElem(key, hashVal);
-    if (!findIt)
-        return 0;
-    return findIt->fData;
+    return findIt ? findIt->fData : 0;
 }
 
-template <class TVal>
-MemoryManager* RefHashTableOf<TVal>::getMemoryManager() const
+template <class TVal, class THasher>
+inline MemoryManager* RefHashTableOf<TVal, THasher>::getMemoryManager() const
 {
     return fMemoryManager;
 }
 
-template <class TVal>
-unsigned int RefHashTableOf<TVal>::getHashModulus() const
+template <class TVal, class THasher>
+inline XMLSize_t RefHashTableOf<TVal, THasher>::getHashModulus() const
 {
     return fHashModulus;
 }
 
-template <class TVal>
-unsigned int RefHashTableOf<TVal>::getCount() const
+template <class TVal, class THasher>
+inline XMLSize_t RefHashTableOf<TVal, THasher>::getCount() const
 {
     return fCount;
 }
@@ -373,8 +382,8 @@ unsigned int RefHashTableOf<TVal>::getCount() const
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Getters
 // ---------------------------------------------------------------------------
-template <class TVal>
-void RefHashTableOf<TVal>::setAdoptElements(const bool aValue)
+template <class TVal, class THasher>
+inline void RefHashTableOf<TVal, THasher>::setAdoptElements(const bool aValue)
 {
     fAdoptedElems = aValue;
 }
@@ -382,17 +391,18 @@ void RefHashTableOf<TVal>::setAdoptElements(const bool aValue)
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Putters
 // ---------------------------------------------------------------------------
-template <class TVal> void RefHashTableOf<TVal>::put(void* key, TVal* const valueToAdopt)
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::put(void* key, TVal* const valueToAdopt)
 {
     // Apply 0.75 load factor to find threshold.
-    unsigned int threshold = fHashModulus * 3 / 4;
-    
+    XMLSize_t threshold = fHashModulus * 3 / 4;
+
     // If we've grown too big, expand the table and rehash.
     if (fCount >= threshold)
         rehash();
 
     // First see if the key exists already
-    unsigned int hashVal;
+    XMLSize_t hashVal;
     RefHashTableBucketElem<TVal>* newBucket = findBucketElem(key, hashVal);
 
     //
@@ -404,14 +414,13 @@ template <class TVal> void RefHashTableOf<TVal>::put(void* key, TVal* const valu
         if (fAdoptedElems)
             delete newBucket->fData;
         newBucket->fData = valueToAdopt;
-		newBucket->fKey = key;
+        newBucket->fKey = key;
     }
     else
     {
-        //newBucket = new (fMemoryManager) RefHashTableBucketElem<TVal>(key, valueToAdopt, fBucketList[hashVal]);
-		newBucket =
+        newBucket =
              new (fMemoryManager->allocate(sizeof(RefHashTableBucketElem<TVal>)))
-             RefHashTableBucketElem<TVal>(key, valueToAdopt, fBucketList[hashVal]);        
+             RefHashTableBucketElem<TVal>(key, valueToAdopt, fBucketList[hashVal]);
         fBucketList[hashVal] = newBucket;
         fCount++;
     }
@@ -422,25 +431,26 @@ template <class TVal> void RefHashTableOf<TVal>::put(void* key, TVal* const valu
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Private methods
 // ---------------------------------------------------------------------------
-template <class TVal> void RefHashTableOf<TVal>::rehash()
+template <class TVal, class THasher>
+void RefHashTableOf<TVal, THasher>::rehash()
 {
-    const unsigned int newMod = fHashModulus * 2;
+    const XMLSize_t newMod = (fHashModulus * 2) + 1;
 
     RefHashTableBucketElem<TVal>** newBucketList =
         (RefHashTableBucketElem<TVal>**) fMemoryManager->allocate
     (
         newMod * sizeof(RefHashTableBucketElem<TVal>*)
-    );//new RefHashTableBucketElem<TVal>*[newMod];
+    );
 
     // Make sure the new bucket list is destroyed if an
     // exception is thrown.
     ArrayJanitor<RefHashTableBucketElem<TVal>*>  guard(newBucketList, fMemoryManager);
 
     memset(newBucketList, 0, newMod * sizeof(newBucketList[0]));
-    
-    
+
+
     // Rehash all existing entries.
-    for (unsigned int index = 0; index < fHashModulus; index++)
+    for (XMLSize_t index = 0; index < fHashModulus; index++)
     {
         // Get the bucket list head for this entry
         RefHashTableBucketElem<TVal>* curElem = fBucketList[index];
@@ -450,8 +460,7 @@ template <class TVal> void RefHashTableOf<TVal>::rehash()
             // Save the next element before we detach this one
             RefHashTableBucketElem<TVal>* const nextElem = curElem->fNext;
 
-            const unsigned int hashVal = fHash->getHashVal(curElem->fKey, newMod, fMemoryManager);
-            assert(hashVal < newMod);
+            const XMLSize_t hashVal = fHasher.getHashVal(curElem->fKey, newMod);
 
             RefHashTableBucketElem<TVal>* const newHeadElem = newBucketList[hashVal];
 
@@ -472,21 +481,21 @@ template <class TVal> void RefHashTableOf<TVal>::rehash()
 
     // Delete the old bucket list.
     fMemoryManager->deallocate(oldBucketList);//delete[] oldBucketList;
-    
+
 }
 
-template <class TVal> RefHashTableBucketElem<TVal>* RefHashTableOf<TVal>::
-findBucketElem(const void* const key, unsigned int& hashVal)
+template <class TVal, class THasher>
+inline RefHashTableBucketElem<TVal>* RefHashTableOf<TVal, THasher>::
+findBucketElem(const void* const key, XMLSize_t& hashVal)
 {
     // Hash the key
-    hashVal = fHash->getHashVal(key, fHashModulus, fMemoryManager);
-    assert(hashVal < fHashModulus);
+    hashVal = fHasher.getHashVal(key, fHashModulus);
 
     // Search that bucket for the key
     RefHashTableBucketElem<TVal>* curElem = fBucketList[hashVal];
     while (curElem)
     {
-		if (fHash->equals(key, curElem->fKey))
+        if (fHasher.equals(key, curElem->fKey))
             return curElem;
 
         curElem = curElem->fNext;
@@ -494,18 +503,18 @@ findBucketElem(const void* const key, unsigned int& hashVal)
     return 0;
 }
 
-template <class TVal> const RefHashTableBucketElem<TVal>* RefHashTableOf<TVal>::
-findBucketElem(const void* const key, unsigned int& hashVal) const
+template <class TVal, class THasher>
+inline const RefHashTableBucketElem<TVal>* RefHashTableOf<TVal, THasher>::
+findBucketElem(const void* const key, XMLSize_t& hashVal) const
 {
     // Hash the key
-    hashVal = fHash->getHashVal(key, fHashModulus, fMemoryManager);
-    assert(hashVal < fHashModulus);
+    hashVal = fHasher.getHashVal(key, fHashModulus);
 
     // Search that bucket for the key
     const RefHashTableBucketElem<TVal>* curElem = fBucketList[hashVal];
     while (curElem)
     {
-        if (fHash->equals(key, curElem->fKey))
+        if (fHasher.equals(key, curElem->fKey))
             return curElem;
 
         curElem = curElem->fNext;
@@ -517,11 +526,11 @@ findBucketElem(const void* const key, unsigned int& hashVal) const
 // ---------------------------------------------------------------------------
 //  RefHashTableOfEnumerator: Constructors and Destructor
 // ---------------------------------------------------------------------------
-template <class TVal> RefHashTableOfEnumerator<TVal>::
-RefHashTableOfEnumerator(RefHashTableOf<TVal>* const toEnum
+template <class TVal, class THasher> RefHashTableOfEnumerator<TVal, THasher>::
+RefHashTableOfEnumerator(RefHashTableOf<TVal, THasher>* const toEnum
                          , const bool adopt
                          , MemoryManager* const manager)
-	: fAdopted(adopt), fCurElem(0), fCurHash((unsigned int)-1), fToEnum(toEnum)
+    : fAdopted(adopt), fCurElem(0), fCurHash((XMLSize_t)-1), fToEnum(toEnum)
     , fMemoryManager(manager)
 {
     if (!toEnum)
@@ -531,21 +540,22 @@ RefHashTableOfEnumerator(RefHashTableOf<TVal>* const toEnum
     //  Find the next available bucket element in the hash table. If it
     //  comes back zero, that just means the table is empty.
     //
-    //  Note that the -1 in the current hash tells it to start from the
-    //  beginning.
+    //  Note that the -1 in the current hash tells it to start
+    //  from the beginning.
     //
     findNext();
 }
 
-template <class TVal> RefHashTableOfEnumerator<TVal>::~RefHashTableOfEnumerator()
+template <class TVal, class THasher>
+RefHashTableOfEnumerator<TVal, THasher>::~RefHashTableOfEnumerator()
 {
     if (fAdopted)
         delete fToEnum;
 }
 
 
-template <class TVal> RefHashTableOfEnumerator<TVal>::
-RefHashTableOfEnumerator(const RefHashTableOfEnumerator<TVal>& toCopy) :
+template <class TVal, class THasher> RefHashTableOfEnumerator<TVal, THasher>::
+RefHashTableOfEnumerator(const RefHashTableOfEnumerator<TVal, THasher>& toCopy) :
     XMLEnumerator<TVal>(toCopy)
     , XMemory(toCopy)
     , fAdopted(toCopy.fAdopted)
@@ -558,7 +568,8 @@ RefHashTableOfEnumerator(const RefHashTableOfEnumerator<TVal>& toCopy) :
 // ---------------------------------------------------------------------------
 //  RefHashTableOfEnumerator: Enum interface
 // ---------------------------------------------------------------------------
-template <class TVal> bool RefHashTableOfEnumerator<TVal>::hasMoreElements() const
+template <class TVal, class THasher>
+bool RefHashTableOfEnumerator<TVal, THasher>::hasMoreElements() const
 {
     //
     //  If our current has is at the max and there are no more elements
@@ -569,7 +580,8 @@ template <class TVal> bool RefHashTableOfEnumerator<TVal>::hasMoreElements() con
     return true;
 }
 
-template <class TVal> TVal& RefHashTableOfEnumerator<TVal>::nextElement()
+template <class TVal, class THasher>
+TVal& RefHashTableOfEnumerator<TVal, THasher>::nextElement()
 {
     // Make sure we have an element to return
     if (!hasMoreElements())
@@ -585,7 +597,8 @@ template <class TVal> TVal& RefHashTableOfEnumerator<TVal>::nextElement()
     return *saveElem->fData;
 }
 
-template <class TVal> void* RefHashTableOfEnumerator<TVal>::nextElementKey()
+template <class TVal, class THasher>
+void* RefHashTableOfEnumerator<TVal, THasher>::nextElementKey()
 {
     // Make sure we have an element to return
     if (!hasMoreElements())
@@ -601,9 +614,10 @@ template <class TVal> void* RefHashTableOfEnumerator<TVal>::nextElementKey()
     return saveElem->fKey;
 }
 
-template <class TVal> void RefHashTableOfEnumerator<TVal>::Reset()
+template <class TVal, class THasher>
+void RefHashTableOfEnumerator<TVal, THasher>::Reset()
 {
-    fCurHash = (unsigned int)-1;
+    fCurHash = (XMLSize_t)-1;
     fCurElem = 0;
     findNext();
 }
@@ -613,7 +627,8 @@ template <class TVal> void RefHashTableOfEnumerator<TVal>::Reset()
 // ---------------------------------------------------------------------------
 //  RefHashTableOfEnumerator: Private helper methods
 // ---------------------------------------------------------------------------
-template <class TVal> void RefHashTableOfEnumerator<TVal>::findNext()
+template <class TVal, class THasher>
+void RefHashTableOfEnumerator<TVal, THasher>::findNext()
 {
     //
     //  If there is a current element, move to its next element. If this
