@@ -16,12 +16,11 @@
  */
 
 /*
- * $Id: XPathException.hpp 568078 2007-08-21 11:43:25Z amassari $
+ * $Id: XPathException.hpp 527149 2007-04-10 14:56:39Z amassari $
  */
 
-
-#if !defined(XPATHEXCEPTION_HPP)
-#define XPATHEXCEPTION_HPP
+#if !defined(XERCESC_INCLUDE_GUARD_XPATHEXCEPTION_HPP)
+#define XERCESC_INCLUDE_GUARD_XPATHEXCEPTION_HPP
 
 #include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/XMLException.hpp>
